# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is RDFGrabber version 1.0.
#
# The Initial Developer of the Original Code is European Environment
# Agency (EEA).  Portions created by EEA are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributor(s):
# Soren Roug, EEA
#
import operator, string, time
try:
    import urllib2 # Support for authenticated proxy
    ulib2 = 1
except:
    import urllib
    ulib2 = 0
import xmllib
import sys
from types import *
from objects import resource, literal
from const import *
from string import join, split

ns_separator = " "

ABOUT_ATTRIBUTE = RDFNS + ns_separator + "about"
ID_ATTRIBUTE = RDFNS + ns_separator + "ID"
RESOURCE_ATTRIBUTE = RDFNS + ns_separator + "resource"
PARSETYPE_ATTRIBUTE = RDFNS + ns_separator + "parseType"

class RDFParser(xmllib.XMLParser):
    """Parse an RDF file"""

    def __init__(self,adder,http_proxy=None):
	self.adder=adder
        if http_proxy:
	    self._proxies = { 'http': http_proxy }
	else:
	    self._proxies = {}
	self.rdf = {}
	self.encoding = 'UTF-8'
	self.subject=[]
	self.predicate=[]
	self.object=None
	self.li_count = 0
	self.anon_count = 0
	self.__data=[]
	self.elements = {
	# RDF
	  RDFNS + ns_separator + 'RDF':
		  (self.start_rdf,self.end_rdf),
	  RDFNS + ns_separator + 'Description':
		  (self.start_description,self.end_description),
	  RDFNS + ns_separator + 'Seq':
		  (self.start_container,self.end_container),
	  RDFNS + ns_separator + 'Bag':
		  (self.start_container,self.end_container),
	  RDFNS + ns_separator + 'Alt':
		  (self.start_container,self.end_container),
	  RDFNS + ns_separator + 'li':
		  (self.start_list,self.end_list),
	  RDFNS + ns_separator + 'value':
		  (self.start_value,self.end_value),
	  }
	xmllib.XMLParser.__init__(self)

    def generate_uri(self):
	self.anon_count = self.anon_count + 1
	return resource("genid:" + str(self.anon_count),anonymous=1)

    def handle_xml(self,encoding,standalone):
	if(encoding):
	    self.encoding = encoding

    def unknown_starttag(self, tag, atts):
	if self.subject == []:
	    self.start_top(tag, atts)
	else:
	    self.start_lower(tag,atts)

    def unknown_endtag(self, tag):
	if len(self.subject) == 1:
	    self.end_top(tag)
	else:
	    self.end_lower(tag)

    def start_lower(self, tag, atts):
	self.__data = []
	self.subject.append(self.subject[-1])
        if atts.has_key(PARSETYPE_ATTRIBUTE):
	    self.subject.pop()
            self.subject.append(self.generate_uri())
        elif atts.has_key(RESOURCE_ATTRIBUTE):
	    if atts[RESOURCE_ATTRIBUTE][0] == "#":
		self.object = resource(self.baseurl + atts[RESOURCE_ATTRIBUTE])
	    else:
		self.object = resource(atts[RESOURCE_ATTRIBUTE])

    def end_lower(self, tag):
	if self.object == None:
	    self.object = literal(string.join(self.__data,""))
	type = resource(join(split(tag, ns_separator), ""))
	self.adder(self.subject.pop(), type, self.object)
	self.object = None

    def start_top(self, tag, atts):
	self.__data = []
	self.anon_object=0
	tag = join(split(tag, ns_separator), "")
        if atts.has_key(ABOUT_ATTRIBUTE):
	    if atts[ABOUT_ATTRIBUTE][0] == "#":
		self.subject.append(resource(self.baseurl +
		  atts[ABOUT_ATTRIBUTE]))
	    else:
		self.subject.append(resource(atts[ABOUT_ATTRIBUTE]))
        elif atts.has_key(ID_ATTRIBUTE):
            self.subject.append(resource(self.baseurl +
	     "#" + atts[ID_ATTRIBUTE]))
        else:
            self.subject.append(self.generate_uri())

	self.adder(self.subject[-1], TYPE, tag)

        for att in atts.keys():
            if att == ABOUT_ATTRIBUTE or att == ID_ATTRIBUTE:
                pass
            else:
                new_att = resource(join(split(att, ns_separator), ""))
                self.adder(self.subject[-1], new_att, literal(atts[att]))

    def end_top(self,tag):
	self.subject.pop()

    def ignore_tag(self,atts=None):
	pass

    def handle_data(self, text):
	self.__data.append(text)

    def handle_cdata(self, text):
        self.__data.append(text)

    def handle_charref(self,ref):
	self.handle_data('&#' + ref + ';')

    def unknown_entityref(self,ref):
	self.handle_data('&' + ref + ';')

    def syntax_error(self,message):
	pass

    def start_rdf(self, tag, atts):
	"Start of rdf info"
	self.rdf = {}

    def end_rdf(self,tag):
	self.rdf['encoding'] = self.encoding

    def start_value(self, tag, atts):
	self.__data = []
        if atts.has_key(RESOURCE_ATTRIBUTE):
	    if atts[RESOURCE_ATTRIBUTE][0] == "#":
		self.object = resource(self.baseurl + atts[RESOURCE_ATTRIBUTE])
	    else:
		self.object = resource(atts[RESOURCE_ATTRIBUTE])
	
    def end_value(self, tag):
	if self.object == None:
	    self.object = literal(string.join(self.__data,""))
	type = resource(join(split(tag, ns_separator), ""))
	self.adder(self.subject[-1], type, self.object)
	self.object = self.subject.pop()
	self.subject.append(self.subject[-1])

    def start_container(self, tag, atts):
	self.subject.append(self.generate_uri())
	self.li_count = 0

    def end_container(self,tag):
	tag = join(split(tag, ns_separator), "")
	self.adder(self.subject.pop(),TYPE,resource(tag))

    def start_description(self, tag, atts):
	self.__data = []
        if atts.has_key(ABOUT_ATTRIBUTE):
	    if atts[ABOUT_ATTRIBUTE][0] == "#":
		self.subject.append(resource(self.baseurl +
		  atts[ABOUT_ATTRIBUTE]))
	    else:
		self.subject.append(resource(atts[ABOUT_ATTRIBUTE]))
        elif atts.has_key(ID_ATTRIBUTE):
            self.subject.append(self.baseurl + "#" + atts[ID_ATTRIBUTE])
        else:
            self.subject.append(self.generate_uri())

        for att in atts.keys():
            if att == ABOUT_ATTRIBUTE or att == ID_ATTRIBUTE:
                pass
            else:
                new_att = resource(join(split(att, ns_separator), ""))
                self.adder(self.subject[-1], new_att, literal(atts[att]))

    def end_description(self,tag):
	self.subject.pop()

    def start_list(self, tag, atts):
	self.__data = []
	self.object = None
	tag = join(split(tag, ns_separator), "")
	self.li_count = self.li_count + 1
	self.predicate.append(resource(RDFNS + '_' + str(self.li_count)))

    def end_list(self, tag):
	if self.object == None:
	    self.object = literal(string.join(self.__data,""))
	self.adder(self.subject[-1], self.predicate.pop(), self.object)
 	self.object = self.subject[-1]

    def handle_starttag(self, tag, method, atts):
        method(tag,atts)

    def handle_endtag(self, tag, method):
        method(tag)

    def parse_url(self, url):
	"""
	Grab the file from the webserver and feed it to the parser
	"""
	self.encoding = 'UTF-8'
	self.subject=[]
	self.predicate=[]
	self.object=None
	self.li_count = 0
	self.__data=[]
	self.baseurl = url
        if ulib2 == 1:
            proxy_support = urllib2.ProxyHandler(self._proxies)
            opener = urllib2.build_opener(proxy_support, urllib2.HTTPHandler)

            urllib2.install_opener(opener)
            f = urllib2.urlopen(url)
        else:
	    try:
		u = urllib.URLopener(proxies=self._proxies)
	    except IOError, e:
		raise IOError, "Unsupported protocol"
	    else:
		u.addheader("User-agent", "RDFGrabber (helpdesk@eionet.eu.int)")
		f = u.open(url)
	if not f:
	    raise IOError, "Failure in open %s" % url
	self.rdfsource = f.read()
	self.feed(self.rdfsource)
	f.close()
